/* 
 * @(#)sockwatch.c v1.0 1999 Rob Thomas robt@cymru.com
 * This program is designed to serve as an alert mechanism, watching a
 * given host and TCP port combination.  This can be handy when you
 * suspect that network connectivity is dropping for very short periods.
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <netinet/in_systm.h>
#include <unistd.h>
#include <fcntl.h>
#include <strings.h>
#include <errno.h>
#include <netinet/ip.h>
#include <arpa/inet.h>
#include <signal.h>

int		port,
		interval;

static void sigHand(int);

static void
sigHand(int signo)
{
	if (signo == SIGPIPE) {
		fprintf(stderr, "SIGPIPE received!!\n");
		exit(5);
	} else {
		fprintf(stdout, "Received signal %d.\n", signal);
		return;
	}
}

void
displayUsage(char *prog)
{
	fprintf(stderr, "%s: usage: %s <hostname> <port> <interval>\n",
			basename(prog), basename(prog));
	exit(-1);
}

void
startWatching(struct in_addr myaddr)
{
	struct sockaddr_in mytcp;
	int			tcpsock,
				retval,
				snd_buf = 8192;
	char buf[BUFSIZ] = "Foo foo foo!";

	if((tcpsock = socket(AF_INET, SOCK_STREAM, 6)) < 0) {
		perror("socket()");
		exit(-1);
	}

	mytcp.sin_addr.s_addr = myaddr.s_addr;
	mytcp.sin_family = AF_INET;
	mytcp.sin_port = htons(port);

	if (( 0 > setsockopt(tcpsock, SOL_SOCKET, SO_SNDBUF, &snd_buf,
			sizeof(snd_buf)))) {
		perror("setsockopt()");
		exit(-1);
	}

	if (( 0 > connect(tcpsock, (struct sockaddr *) &mytcp, sizeof(mytcp)))) {
		perror("connect()");
	}

	fprintf(stdout, "connect() successful...\n");

	for (;;) {
		retval = send(tcpsock, buf, sizeof(buf), 0);
		if (sizeof(buf) != retval) {
			perror("send()");
			exit(10);
		}

		retval = recv(tcpsock, buf, sizeof(buf), 0);

		if (0 > retval) {
			perror("recv()");
			exit(10);
		}

		sleep(30);
	}
}
	  
struct in_addr
resolvHost(char *address)
{
	struct in_addr myaddr;
	struct hostent *host;

	if((myaddr.s_addr = inet_addr(address)) == -1) {
		if((host = gethostbyname(address)) == NULL)
			fprintf(stderr,"%s Invalid address\n",address);
		else {
			bcopy((int *) * &host->h_addr,&myaddr.s_addr,host->h_length); 
			return myaddr;
		}
	}

    return myaddr;
}

int
main(int argc, char **argv)
{
	struct in_addr myaddr;
  
	if(argc < 3) {
		displayUsage(argv[0]);
		exit(-1);
	}

	if (signal(SIGPIPE, sigHand) == SIG_ERR) {
		perror("signal()");
		exit(-1);
	}

	port = atoi(argv[2]);
	myaddr = resolvHost(argv[1]);

	startWatching(myaddr);

	exit(0);
}

